/**
 * TmoteSense permite visualizar los valores medidos de los sensores de luz
 * y temperatura de un nodo remoto.
 *
 * Lee los datos desde el puerto USB, y guarda 1 de cada 20 mediciones en
 * una base de datos.
 *
 * Además verifica que los datos no superen un umbral definido por el usuario.
 * En caso contrario envía un email al correo definido en la clase GoogleTest,
 * o el correo definido en la base de datos.
 *
 *
 * versión original de Alejandro Merello
 *
 *   Modificado por
 *      Carlos Apablaza B
 *
 *                       Diciembre de 2010
 */

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.mail.MessagingException;
import javax.swing.*;
import javax.imageio.*;
import java.io.*;

import net.tinyos.message.*;
import net.tinyos.packet.*;
import net.tinyos.util.*;

import java.sql.*;

public class TmoteSense extends JComponent implements MessageListener {
  static TmoteSensePanel panel = new TmoteSensePanel();               // JPanel con el contenido

  public int k=0;

  public void setk0() {
    k=0;
  }

  public void setk() {
    k++;
  }

  private MoteIF moteIF;    /* MoteIF provee una interfaz a nivel de aplicacion para enviar y recibir
                               mensajes a traves del puerto serial, conexion TCP u otro tipo de conexion */

  // Constructor
  public TmoteSense(MoteIF moteIF) {
    this.moteIF = moteIF;                                     // Inicializa la interfaz
    this.moteIF.registerListener(new TmoteSenseMsg(), this);  // Registra un Listener para mensajes de tipo TmoteSenseMsg
  }

  //Este metodo es invocado tras la recepcion exitosa de un paquete
  public void messageReceived(int to, Message message) {

    TmoteSenseMsg msg = (TmoteSenseMsg) message; // Crea un nuevo objeto mensaje con la estructura de TmoteSenseMsg (Generado por mig)
    panel.setCounter(msg.get_counter());         // Actualiza el estado del programa remoto en el manejador de la ventana
    if(msg.get_counter()==0) {                   // Dependiendo del estado actualiza la variable correspondiente
      panel.setTemperature(msg.get_temperature());
      panel.setLight(msg.get_light());

/////////////// DB ///////////////

      if(k==20) {  // Se reciben mediciones cada 1 s. Guardar en DB cada 20 s.
        Connection con = null;
        PreparedStatement stmt = null;

        // Definimos el driver y la url
        String sDriver = "com.mysql.jdbc.Driver";
        String sURL = "jdbc:mysql://localhost:3306/tmotesky";

        int temp=msg.get_temperature();
        int lux=msg.get_light();

        // Cargamos el driver y realizamos la conexion
        try {
          Class.forName(sDriver).newInstance();
          con = DriverManager.getConnection(sURL,"user","user");
          stmt = con.prepareStatement("INSERT INTO temp VALUES (NULL ," + temp + ",CURRENT_TIMESTAMP);");
          stmt.executeUpdate();
          System.out.println("insertando en temp " + temp);

	  Class.forName(sDriver).newInstance();
          con = DriverManager.getConnection(sURL,"user","user");
          stmt = con.prepareStatement("INSERT INTO lux VALUES (NULL ," + lux + ",CURRENT_TIMESTAMP);");
          stmt.executeUpdate();
          System.out.println("insertando en lux " + lux);
        }
        catch (SQLException sqle) {
          System.out.println("SQLState: " + sqle.getSQLState());
          System.out.println("SQLErrorCode: " + sqle.getErrorCode());
          sqle.printStackTrace();

        }
        catch (Exception e) {
          e.printStackTrace();
        }
        finally {
          if (con != null) {
            try {
              stmt.close();
              con.close();
            }
	    catch(Exception e){
              e.printStackTrace();
            }
          }
        }
        setk0();
      }
      setk();

/////////////////////////////
    }
    panel.repaint(); // Refresca el JPanel
  }

  // Metodo principal
  public static void main(String[] args) throws Exception {

    panel.addEtiquetas();

    String source = null;
    if (args.length == 2) { // Este bloque condicional termina el programa si se ingresa con parametros incorrectos
      if (!args[0].equals("-comm")) {
	      usage();
        System.exit(1);
      }
      source = args[1];
    }
    else if (args.length != 0) {
      usage();
      System.exit(1);
    }

    JFrame.setDefaultLookAndFeelDecorated(true);
    JFrame frame = new JFrame("Tmote Sense");
    JLabel label = new JLabel("Medicion Remota",JLabel.CENTER);
    frame.getContentPane().add(label,BorderLayout.NORTH);
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame.add(panel);
    frame.pack();                // Comprime el tamano de la ventana al minimo posible
    frame.setVisible(true);

    PhoenixSource phoenix;      /* PhoenixSource se construye sobre un PacketSource para proveer: lectura y
                                   escritura de paquetes automaticas, reinicio automatico. */

    if (source == null)         // MoteIF se construye sobre una PhoenixSource
      phoenix = BuildSource.makePhoenix("serial@/dev/ttyUSB0:tmote", PrintStreamMessenger.err);
    else
      phoenix = BuildSource.makePhoenix(source, PrintStreamMessenger.err);

    MoteIF mif = new MoteIF(phoenix);         // Crea una nueva interfaz mote con la fuente de paquetes phoenix.
    TmoteSense serial = new TmoteSense(mif);  // Invoca al constructor
  }

  private static void usage() {
    System.err.println("usage: TmoteSense [-comm <source>]");
  }
}

// Manejador de la interfaz de usuario del programa
class TmoteSensePanel extends JPanel {
  int c=5;
  int counter,lux,temp;
  private JLabel l1, l2, l3;
  public JTextField campoTexto1, campoTexto2, campoTexto3;
  private Container contenedor;
  public boolean flagtemp;
  public java.util.Timer timer = new java.util.Timer();

  TimerTask ttask;

  public boolean flag=true;

  private String text1="0", text2="0", text3="email";

  ResultSet rs = null;

  private int UmbralTemp=-1;
  private int UmbralLux=-1;
  private String emaildb;

  // Constructor de la UI
  public TmoteSensePanel() {
    setBorder(BorderFactory.createLineBorder(Color.black));
    counter=lux=temp=0;
  }

  public Dimension getPreferredSize() {
    return new Dimension(600, 380);
  }

  public void addEtiquetas() {
    l1 = new JLabel("Umbral de Temperatura: ");
    l2 = new JLabel("Umbral de Iluminancia: ");
    l3 = new JLabel("Email destinatario: ");

    // Definimos el driver y la url
    String login = "user";
    String password = "user";
    String url = "jdbc:mysql://localhost:3306/tmotesky";
    Connection conn = null;
    Statement statement = null;
    ResultSet rs = null;

    // Cargamos el driver y realizamos la conexion
    try {
      Class.forName("com.mysql.jdbc.Driver").newInstance ();
      conn = DriverManager.getConnection(url, login, password);
      statement = conn.createStatement();

      rs = statement.executeQuery("SELECT * FROM umbral_temp");
      if(rs.next()) text1 = rs.getString("utemp"); System.out.println("Umbral Temperatura: " + rs.getString("utemp"));

      rs = statement.executeQuery("SELECT * FROM umbral_lux");
      if(rs.next()) text2 = rs.getString("ulux"); System.out.println("Umbral Iluminancia " + rs.getString("ulux"));

      rs = statement.executeQuery("SELECT * FROM emaildst");
      if(rs.next()) text3 = rs.getString("email"); System.out.println("Email Destino: " + rs.getString("email"));
    }
    catch (SQLException sqle) {
      System.out.println("SQLState: " + sqle.getSQLState());
      System.out.println("SQLErrorCode: " + sqle.getErrorCode());
      sqle.printStackTrace();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    finally {
      if (conn != null) {
        try {
          conn.close();
        }
        catch(Exception e){
          e.printStackTrace();
        }
      }
    }

    add(l1);
    campoTexto1 = new JTextField(String.valueOf(text1), 10 );
    add(campoTexto1);

    add(l2);
    campoTexto2 = new JTextField(String.valueOf(text2), 10 );
    add(campoTexto2);

    add(l3);
    campoTexto3 = new JTextField(String.valueOf(text3), 20 );
    add(campoTexto3);

    ManejadorCampoTexto manejador = new ManejadorCampoTexto();
    campoTexto1.addActionListener( manejador );
    campoTexto2.addActionListener( manejador );
    campoTexto3.addActionListener( manejador );
  }

  public void Verify() throws MessagingException {

    String login = "user";
    String password = "user";
    String url = "jdbc:mysql://localhost:3306/tmotesky";
    Connection conn = null;
    Statement statement = null;
    ResultSet rs2 = null;

    // Cargamos el driver y realizamos la conexion
    try {
      Class.forName("com.mysql.jdbc.Driver").newInstance ();
      conn = DriverManager.getConnection(url, login, password);
      statement = conn.createStatement();

      rs2 = statement.executeQuery("SELECT * FROM umbral_temp");
      if(rs2.next()) UmbralTemp = Integer.parseInt(rs2.getString("utemp"));

      rs2 = statement.executeQuery("SELECT * FROM umbral_lux");
      if(rs2.next()) UmbralLux = Integer.parseInt(rs2.getString("ulux"));

      rs2 = statement.executeQuery("SELECT * FROM emaildst");
      if(rs2.next()) emaildb = rs2.getString("email");

      if(c==0) {  // Esperar 5 segundos antes de actualizar, si no cuesta hacer cambios con teclado
        // Actualizar JTextFields
        campoTexto1.setText(String.valueOf(UmbralTemp));
        campoTexto2.setText(String.valueOf(UmbralLux));
        campoTexto3.setText(emaildb);
        c=5;
      }
      c--;
    }
    catch (SQLException sqle) {
      System.out.println("SQLState: " + sqle.getSQLState());
      System.out.println("SQLErrorCode: " + sqle.getErrorCode());
      sqle.printStackTrace();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    finally {
      if (conn != null) {
        try {
          conn.close();
        }
        catch(Exception e){
          e.printStackTrace();
        }
      }
    }

    if((UmbralTemp !=-1) && (temp>UmbralTemp) && (flag==true)) {

      flag=false;
      timer.schedule(new RemindTask(), 60 * 1000);

      System.out.println("Se ha superado el umbral de temperatura");

      // Echar a andar un timer de 60 segundos antes de enviar otro correo
      GoogleTest email1 = new GoogleTest();

      String emailMsgTxt = "Se ha sobrepasado el umbral de temperatura. INCENDIO INMINENTE!!" +
            "\n\nTemperatura actual: " + temp + " C" +
            "\nIluminancia Actual: " +lux + "[lux]" +
            "\n\nPD: Bomberos: 132";
      String emailSubjectTxt = "Umbral sobrepasado";
      String emailFromAddress = "sensor@sensorservice.com";
      String[] sendTo = { emaildb };

      email1.sendSSLMessage(sendTo, emailSubjectTxt, emailMsgTxt, emailFromAddress);

      System.out.println("Se ha enviado un correo a: " + emaildb);

    }

    if((UmbralLux !=-1) && (lux>UmbralLux) && (flag==true)) {

      flag=false;
      java.util.Timer timerlux = new java.util.Timer();
      timerlux.schedule(new RemindTask(), 60 * 1000);

      System.out.println("Se ha superado el umbral de iluminancia");

      GoogleTest email2 = new GoogleTest();

      String emailMsgTxt = "Se ha sobrepasado el umbral de iluminancia. INCENDIO INMINENTE!!" +
            "\n\nTemperatura actual: " + temp + " C" +
            "\nIluminancia Actual: " +lux + "[lux]" +
            "\n\nPD: Bomberos: 132";
      String emailSubjectTxt = "Umbral sobrepasado";
      String emailFromAddress = "sensor@sensorservice.com";
      String[] sendTo = { emaildb };

      email2.sendSSLMessage(sendTo, emailSubjectTxt, emailMsgTxt, emailFromAddress);

      System.out.println("Se ha enviado un correo a: " + emaildb);

    }

  }
  /*
   * Método run se ejecuta al terminar el timer de 60 segundos
   */
  class RemindTask extends TimerTask {
    public void run() {
      flag=true;
    }
  }

  // clase interna privada para el manejo de eventos
   private class ManejadorCampoTexto implements ActionListener {
      // procesar eventos de campo de texto
      public void actionPerformed( ActionEvent evento ) {
         String cadena = "";

         // El usuario oprimio Intro en objeto JTextField campoTexto1
         if ( evento.getSource() == campoTexto1 ) {
           UmbralTemp = Integer.parseInt(evento.getActionCommand());
           cadena = "Nuevo Umbral de Temperatura: " + UmbralTemp + " C";

           Connection con = null;
           PreparedStatement stmt = null;

           // Definimos el driver y la url
           String sDriver = "com.mysql.jdbc.Driver";
           String sURL = "jdbc:mysql://localhost:3306/tmotesky";

           // Cargamos el driver y realizamos la conexion
           try {
             Class.forName(sDriver).newInstance();
             con = DriverManager.getConnection(sURL,"user","user");
             stmt = con.prepareStatement ("UPDATE umbral_temp SET utemp =" + UmbralTemp + ", time=CURRENT_TIMESTAMP;");
             stmt.executeUpdate();
             System.out.println("Nuevo umbral de temperatura " + UmbralTemp);
           }
           catch (SQLException sqle) {
             System.out.println("SQLState: " + sqle.getSQLState());
             System.out.println("SQLErrorCode: " + sqle.getErrorCode());
             sqle.printStackTrace();
           }
           catch (Exception e) {
             e.printStackTrace();
           }
           finally {
             if (con != null) {
               try {
                 stmt.close();
                 con.close();
               }
               catch(Exception e){
                 e.printStackTrace();
               }
             }
           }
         }

         // El usuario oprimio Intro en objeto JTextField campoTexto2
         else if ( evento.getSource() == campoTexto2 ) {
           UmbralLux = Integer.parseInt(evento.getActionCommand());
           cadena = "Nuevo Umbral de Iluminancia: " + UmbralLux + " [lux]";

           Connection con = null;
           PreparedStatement stmt = null;

           // Definimos el driver y la url
           String sDriver = "com.mysql.jdbc.Driver";
           String sURL = "jdbc:mysql://localhost:3306/tmotesky";

           // Cargamos el driver y realizamos la conexion
           try {
             Class.forName(sDriver).newInstance();
             con = DriverManager.getConnection(sURL,"user","user");
             stmt = con.prepareStatement ("UPDATE umbral_lux SET ulux = " + UmbralLux + ", time=CURRENT_TIMESTAMP;");
             stmt.executeUpdate();
             System.out.println("nuevo umbral de iluminancia " + UmbralLux);

           }
           catch (SQLException sqle) {
             System.out.println("SQLState: " + sqle.getSQLState());
             System.out.println("SQLErrorCode: " + sqle.getErrorCode());
             sqle.printStackTrace();
           }
           catch (Exception e) {
             e.printStackTrace();
           }
           finally {
             if (con != null) {
               try {
                 stmt.close();
                 con.close();
               }
               catch(Exception e){
                 e.printStackTrace();
               }
             }
           }
         }

       else if ( evento.getSource() == campoTexto3 ) {
         emaildb = evento.getActionCommand();
         cadena = "Nuevo Email destinatario: " + emaildb;

         Connection con = null;
         PreparedStatement stmt = null;

         // Definimos el driver y la url
         String sDriver = "com.mysql.jdbc.Driver";
         String sURL = "jdbc:mysql://localhost:3306/tmotesky";

         // Cargamos el driver y realizamos la conexion
         try {
           Class.forName(sDriver).newInstance();
           con = DriverManager.getConnection(sURL,"user","user");
           stmt = con.prepareStatement ("UPDATE emaildst SET email ='" + emaildb + "', time=CURRENT_TIMESTAMP;");
           stmt.executeUpdate();
           System.out.println("nuevo correo destinatario " + emaildb);
         }
         catch (SQLException sqle) {
           System.out.println("SQLState: " + sqle.getSQLState());
           System.out.println("SQLErrorCode: " + sqle.getErrorCode());
           sqle.printStackTrace();
         }
         catch (Exception e) {
           e.printStackTrace();
         }
         finally {
           if (con != null) {
             try {
               stmt.close();
               con.close();
             }
             catch(Exception e){
               e.printStackTrace();
             }
           }
         }
       }
       JOptionPane.showMessageDialog( null, cadena, "Cambio exitoso", JOptionPane.CANCEL_OPTION);
     }
   }


  public void paintComponent(Graphics g) {
    super.paintComponent(g);
    g.setColor(Color.white);g.fillRect(0, 0, 600, 380);g.setColor(Color.black); // Fondo blanco

    Image img = new ImageIcon("sun.jpg").getImage(); // En bimg queda la imagen en formato BufferedImage


    BufferedImage bimg = new BufferedImage(img.getWidth(null), img.getHeight(null), BufferedImage.TYPE_INT_RGB);
    Graphics2D ge = bimg.createGraphics(); ge.drawImage(img, 0, 0, null);

    RescaleOp op = new RescaleOp(1.0f*((float)lux)/1000.0f, 0, null); // Modifica el brillo de la imagen de acuerdo al nivel de luz
    bimg = op.filter(bimg, null);
    
        try {
            Verify();
        } catch (MessagingException ex) {
            Logger.getLogger(TmoteSensePanel.class.getName()).log(Level.SEVERE, null, ex);
        }

    // Dependiendo del estado del nodo remoto dibuja lo que corresponda
    if(counter==0) {
      g.drawImage(bimg, 400-bimg.getWidth(null)/2, 200-30+30-bimg.getHeight(null)/2+20, this);
      g.drawString("Iluminancia: "+lux,450-100,45-18+30+20);
      g.setColor(Color.red);
      g.drawRoundRect(194-100, 100-50+30+20, 11, 200,11,10);
      g.fillRect(195-100, 300-2*temp-50+30+20, 10, 2*temp);
      g.fillOval(180-100,295-50+30+20,40,40);
      g.setColor(Color.black);
      g.drawString("Temperatura",165-102,45-18+30+20);
      g.drawString(temp+" C",188-100,320-50+30+20);
    }
    else if(counter==1) System.out.println("You must switch the sensor node state to SENSOR (red led).");
    g.finalize();
  }

  public void setCounter(int count) {
    counter=count;
  }

  public void setTemperature(int temperature) {
    temp=temperature;
  }

  public void setLight(int light) {
    lux=light;
  }
}
